@@ -1,10 +1,9 @@
 from dataclasses import dataclass
-from http.cookies import SimpleCookie
 from typing import Optional, Any, Dict
 
 from aiohttp.typedefs import RawHeaders
 
-from glQiwiApi.utils.errors import NetworkError
+from glQiwiApi.utils.exceptions import NetworkError
 
 
 @dataclass
@@ -29,13 +28,11 @@ class Response:
 
 
 @dataclass
-class WrapperData:
-    """object: WrapperData"""
-
+class WrappedRequestPayload:
     headers: Dict[Any, Any]
+    json: Dict[Any, Any]
     data: Optional[Dict[Any, Any]] = None
-    json: Optional[Dict[Any, Any]] = None
     cookies: Optional[Dict[Any, Any]] = None
 
 
-__all__ = ("Response", "WrapperData")
+__all__ = ("Response", "WrappedRequestPayload")
