@@ -1,14 +1,15 @@
 from typing import Optional, Union
 
-from glQiwiApi.types.qiwi_types import currency_parsed as cur
+from glQiwiApi.types import _currencies as currencies  # noqa
+from glQiwiApi.types.amount import CurrencyModel
 
 
 class Currency:
     """
     Class with many currencies
-    >>> usd = Currency.get('840')
+import glQiwiApi.types.basics    >>> usd = Currency.get('840')
     >>> usd
-    ... cur.CurrencyModel(code='USD', decimal_digits=2, name='US Dollar', name_plural='US dollars',
+    ... CurrencyModel(code='USD', decimal_digits=2, name='US Dollar', name_plural='US dollars',
     ...                   rounding=0, symbol='$', symbol_native='$')
 
     >>> usd.symbol
@@ -16,7 +17,7 @@ class Currency:
     """
 
     @classmethod
-    def get(cls, currency_code: Union[str, int]) -> Optional[cur.CurrencyModel]:
+    def get(cls, currency_code: Union[str, int]) -> Optional[CurrencyModel]:
         """
         Implements class-based getitem behaviour
 
@@ -29,8 +30,8 @@ class Currency:
         """
         try:
             if isinstance(currency_code, int) or currency_code.isdigit():
-                return cur.described.get(cur.codes_number[str(currency_code)])
+                return currencies.described.get(currencies.codes_number[str(currency_code)])
             else:
-                return cur.described.get(currency_code.upper())
+                return currencies.described.get(currency_code.upper())
         except (KeyError, AssertionError):
             raise RuntimeError(f"Currency code `{currency_code}` was not found")
